#version 450

//incoming data
layout(location = 0) in vec3 passPosition;
layout(location = 1) in vec3 passNormal;
layout(location = 2) in vec2 passUVCoord;

layout(location = 3) in vec3 passReflectionNormal;

layout(set=0, binding=0) uniform matrixBuffer {
	mat4 viewMatrix;
	mat4 projectionMatrix;
};

struct Material {
	vec3 diffuse;
	float reflectionStrength;
	vec3 specular;
	float shininess;
};

layout(set=0, binding=1) uniform materialBuffer {
	Material mat;
};

struct Light {
	vec4 pos; // pos.w = 0 (dir light), pos.w = 1 (point light)
	vec3 col;
	float spot_exponent;
	vec3 spot_direction;
	float spot_cutoff; // no spotlight if cutoff = 0
};

layout(set=0, binding=2) uniform lightBuffer {
	Light light;
	vec3 lightAmbient;
};

layout(set=0, binding=3) uniform textureCube cubeMapTexture;
layout(set=0, binding=4) uniform sampler cubeMapSampler;

layout(location = 0) out vec4 fragmentColor;
 
void main() {
	///////////////////////////////////////////////////////
	///////////////// Cube Map Reflection /////////////////
	///////////////////////////////////////////////////////
	vec3 pos = normalize(passPosition);
	vec3 normal = normalize(passReflectionNormal);
	vec3 reflected = reflect(pos, normal);
	reflected = vec3(inverse(viewMatrix) * vec4(reflected, 0.0));
	
	vec3 inNormal = normalize(passNormal);
	
	//Diffuse
	vec3 lightVector;
	vec3 light_camcoord = (viewMatrix * light.pos).xyz;
    if (light.pos.w > 0.001f)
		lightVector = normalize(light_camcoord - passPosition);
    else
		lightVector = normalize(light_camcoord);
    float cos_phi = max(dot(inNormal, lightVector), 0.0f);
	
    // Specular
    vec3 eye = normalize(-passPosition);
    vec3 reflection = normalize(reflect(-lightVector, inNormal));
    float cos_psi_n = pow(max(dot(reflection, eye), 0.0f), mat.shininess);
	
	//Spotlight
	float spot = 1.0;
	if (light.spot_cutoff >= 0.001f) {
		float cos_phi_spot = max(dot(-lightVector, mat3(viewMatrix) * light.spot_direction), 0.0f);
		if (cos_phi_spot >= cos(light.spot_cutoff)) {
			spot = pow(cos_phi_spot, light.spot_exponent);
		} else {
			spot = 0.0f;
		}
	}
	
	//Color
	vec3 diffuse_color = mat.diffuse;
	
	//All together
	fragmentColor.rgb = diffuse_color * lightAmbient;
	fragmentColor.rgb += spot * diffuse_color * cos_phi * light.col;
	fragmentColor.rgb += spot * mat.specular * cos_psi_n * light.col;
	fragmentColor *= (1.0 - mat.reflectionStrength);
	fragmentColor += texture(samplerCube(cubeMapTexture, cubeMapSampler), reflected) * mat.reflectionStrength;
	fragmentColor.a = 1.0;
}
