#version 450

layout(location = 0) in vec3 passPosition;
layout(location = 1) in vec3 passNormal;
layout(location = 2) in vec3 passColor;

layout(location = 0) out vec4 fragmentColor;

void main() {
    // set fix light position
    vec3 lightPosition = vec3(0, 10, 0);
    // compute the light vector as the normalized vector between
    // the vertex position and the light position:
    vec3 lightVector = normalize(lightPosition - passPosition);

    // compute the eye vector as the normalized negative vertex position in camera coordinates:
    vec3 eye = normalize(-passPosition);

    // compute the normalized reflection vector using GLSL's built-in reflect() function:
    vec3 reflection = normalize(reflect(-lightVector, normalize(passNormal)));

    // variables used in the phong lighting model:
    float phi = max(dot(passNormal, lightVector), 0);
    float psi = pow(max(dot(reflection, eye), 0), 15);

    vec3 ambientColor = vec3(0.3, 0.2, 0.2);
    vec3 specularColor = vec3(1.0, 1.0, 1.0);

    fragmentColor = vec4(
        ambientColor +
        phi * passColor +
        psi * specularColor,
        1
    );
}