#version 450

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;

layout(set=0, binding=0) uniform matrixBuffer {
    mat4 viewMatrix;
    mat4 projectionMatrix;
};

struct InstanceParams {
    vec3 offset;
    float pad0;
    vec3 color;
    float pad1;
};

layout(set=0, binding=1, std430) readonly buffer instanceBuffer {
    InstanceParams params [];
};

layout(location = 0) out vec3 passPosition;
layout(location = 1) out vec3 passNormal;
layout(location = 2) out vec3 passColor;

void main() {
    vec4 pos = vec4(position + params[gl_InstanceIndex].offset, 1);

    gl_Position = projectionMatrix * viewMatrix * pos;

    // transform the position correctly into view space
    // and pass it to the fragment shader
    passPosition = (viewMatrix * pos).xyz;

    // transform the normal correctly into view space
    // and pass it to the fragment shader
    mat3 normalMatrix = mat3(transpose(inverse(viewMatrix)));
    passNormal = normalize(normalMatrix * normal);
    
    passColor = params[gl_InstanceIndex].color;
}
