#version 450

layout(set=0, binding=5) uniform texture2D	colorTexture;
layout(set=0, binding=6) uniform sampler	colorSampler;

layout(set=1, binding=0) uniform texture2D	normalTexture;

struct Material {
	vec3 diffuse;
	float alpha;
	vec3 specular;
	float shininess;
};

layout(set=0, binding=1) uniform materialBuffer {
	Material mat;
};

struct Light {
	vec4 pos; // pos.w = 0 (dir light), pos.w = 1 (point light)
	vec3 col;
	float spot_exponent; 
	vec3 spot_direction; 
	float spot_cutoff; // no spotlight if cutoff = 0
};

layout(set=0, binding=2) uniform lightBuffer {
	Light light;
	vec3 lightAmbient;
};

layout(set=0, binding=4) uniform texturePropsBuffer {
	int useColorTexture;
};

layout(location = 0) in vec3 passPosition;
layout(location = 1) in vec3 passNormal;
layout(location = 2) in vec3 passTangent;
layout(location = 3) in vec2 passUV;

layout(location = 0) out vec4 fragmentColor;

void main(){
    vec3 normal = normalize(passNormal);
    vec3 tangent = normalize(passTangent);
    vec3 bitangent = cross(tangent, normal);

    /***************** Diffuse ******************************************************/
    // local normal, in tangent space
    vec3 normalWeights = texture(sampler2D(normalTexture, colorSampler), passUV).rgb * 2.0 - 1.0;

    mat3 tbn = transpose(mat3(
        tangent,
        bitangent,
        normal
    ));

    normal = tbn * normalWeights;

    vec3 lightVector;
    vec3 light_camcoord = (light.pos).xyz;
    if (light.pos.w > 0.001f) {
        lightVector = normalize(light_camcoord - passPosition);
    } else {
        lightVector = normalize(light_camcoord);
    }

    //compute the diffuse lighting factor
    float cos_phi = max(dot(normal, lightVector), 0);

    /***************** Specular *****************************************************/
    // compute the normalized reflection vector using GLSL's built-in reflect() function
    vec3 reflection = normalize(reflect(-lightVector, normal));

    // eye vector in tangent space
    vec3 eyeVector = normalize(-passPosition);

    // compute the specular lighting factor
    float cos_psi_n = pow(max(dot(reflection, eyeVector), 0.0f), mat.shininess);

    /***************** Material properties ******************************************/
    vec3 diffuse_color;
    if (useColorTexture != 0)
        diffuse_color = texture(sampler2D(colorTexture, colorSampler), passUV).rgb;
    else
        diffuse_color = mat.diffuse;
    
    /***************** All together *************************************************/
    fragmentColor.rgb = diffuse_color * lightAmbient;
    fragmentColor.rgb += diffuse_color * cos_phi * light.col;
    fragmentColor.rgb += mat.specular * cos_psi_n * light.col;
    fragmentColor.a = 1.0;

}
