#version 450

layout(location = 0) in vec3 Position;
layout(location = 1) in vec3 Normal;
layout(location = 2) in vec2 Tcoord;

layout(set=0, binding=0) uniform matrixBuffer {
	mat4 modelMatrix;
	mat4 viewMatrix;
	mat4 projectionMatrix;
};

layout(location = 0) out vec3 passPosition;
layout(location = 1) out vec3 passNormal;
layout(location = 2) out vec2 passTcoord;

void main() {
	gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(Position, 1);

	//transform the position correctly into view space
	//and pass it to the fragment shader
	passPosition = (viewMatrix * modelMatrix * vec4(Position, 1)).xyz;

	//transform the normal correctly into view space 
	//and pass it to the fragment shader
	mat3 normalMatrix = mat3(transpose(inverse(viewMatrix * modelMatrix)));
	passNormal = normalize(normalMatrix * Normal);
	passTcoord = Tcoord;
}
