#version 450

layout(location = 0) in vec4 passPosition;
layout(location = 1) in vec4 midPosition;

layout(location = 2) in vec4 color;

layout(set=0, binding=0) uniform matrixBuffer {
	mat4 viewMatrix;
	mat4 projectionMatrix;
};

layout(set=0, binding=1) uniform texture2D positionTexture;
layout(set=0, binding=2) uniform texture2D normalTexture;
layout(set=0, binding=3) uniform sampler texSampler;

layout(set=0, binding=4) uniform resolutionBuffer {
	vec2 resolution;
};

layout(location = 0) out vec4 lightingOutput;

void main() {
	vec2 uv = gl_FragCoord.xy / resolution;
    vec4 pos = texture(sampler2D(positionTexture, texSampler), uv);
	vec3 nor = texture(sampler2D(normalTexture, texSampler), uv).xyz;
	
	if (pos.w < 0.5) {
		discard;
	}
	
	float diffuseStrength = max(2.0f - length(pos - midPosition), 0.0f);
	float specularStrength = clamp(diffuseStrength * diffuseStrength, 0.0f, 1.0f);

	if (diffuseStrength < 0) {
		discard;
	}

    vec3 nPosToLight = normalize(vec3(midPosition.xyz - pos.xyz));

    vec3  reflection = normalize(reflect(-nPosToLight,nor.xyz));
    float diffuse = max(dot(nor.xyz, nPosToLight), 0);
    float specular = pow(max(dot(reflection, -normalize(pos.xyz)),0),50);
	
	lightingOutput = vec4(passPosition.xyz - midPosition.xyz, 1.0);
	lightingOutput = vec4(nor.xyz, 1.0);
	
    lightingOutput = color * (
		diffuseStrength * vec4(vec3(diffuse), 1) * 0.7 +
		specularStrength * vec4(1) * specular
	);
}

